import math

class Tween:
    @staticmethod
    def Linear(t: float, b: float, c: float, d: float) -> float:
        """
        线性缓动函数
        :param t: 当前时间
        :param b: 初始值
        :param c: 变化量
        :param d: 持续时间
        :return: 计算后的数值
        """
        return (c * t) / d + b

    class Quad:
        @staticmethod
        def easeIn(t: float, b: float, c: float, d: float) -> float:
            """
            二次方缓动函数（入）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d
            return c * t * t + b

        @staticmethod
        def easeOut(t: float, b: float, c: float, d: float) -> float:
            """
            二次方缓动函数（出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d
            return -c * t * (t - 2) + b

        @staticmethod
        def easeInOut(t: float, b: float, c: float, d: float) -> float:
            """
            二次方缓动函数（入出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d / 2
            if t < 1:
                return (c / 2) * t * t + b
            t -= 1
            return (-c / 2) * (t * (t - 2) - 1) + b

    class Cubic:
        @staticmethod
        def easeIn(t: float, b: float, c: float, d: float) -> float:
            """
            三次方缓动函数（入）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d
            return c * t * t * t + b

        @staticmethod
        def easeOut(t: float, b: float, c: float, d: float) -> float:
            """
            三次方缓动函数（出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t = t / d - 1
            return c * (t * t * t + 1) + b

        @staticmethod
        def easeInOut(t: float, b: float, c: float, d: float) -> float:
            """
            三次方缓动函数（入出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d / 2
            if t < 1:
                return (c / 2) * t * t * t + b
            t -= 2
            return (c / 2) * (t * t * t + 2) + b

    class Quart:
        @staticmethod
        def easeIn(t: float, b: float, c: float, d: float) -> float:
            """
            四次方缓动函数（入）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d
            return c * t * t * t * t + b

        @staticmethod
        def easeOut(t: float, b: float, c: float, d: float) -> float:
            """
            四次方缓动函数（出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t = t / d - 1
            return -c * (t * t * t * t - 1) + b

        @staticmethod
        def easeInOut(t: float, b: float, c: float, d: float) -> float:
            """
            四次方缓动函数（入出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d / 2
            if t < 1:
                return (c / 2) * t * t * t * t + b
            t -= 2
            return (-c / 2) * (t * t * t * t - 2) + b

    class Quint:
        @staticmethod
        def easeIn(t: float, b: float, c: float, d: float) -> float:
            """
            五次方缓动函数（入）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d
            return c * t * t * t * t * t + b

        @staticmethod
        def easeOut(t: float, b: float, c: float, d: float) -> float:
            """
            五次方缓动函数（出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t = t / d - 1
            return c * (t * t * t * t * t + 1) + b

        @staticmethod
        def easeInOut(t: float, b: float, c: float, d: float) -> float:
            """
            五次方缓动函数（入出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            t /= d / 2
            if t < 1:
                return (c / 2) * t * t * t * t * t + b
            t -= 2
            return (c / 2) * (t * t * t * t * t + 2) + b

    class Sine:
        @staticmethod
        def easeIn(t: float, b: float, c: float, d: float) -> float:
            """
            正弦缓动函数（入）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            return -c * math.cos((t / d) * (math.pi / 2)) + c + b

        @staticmethod
        def easeOut(t: float, b: float, c: float, d: float) -> float:
            """
            正弦缓动函数（出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            return c * math.sin((t / d) * (math.pi / 2)) + b

        @staticmethod
        def easeInOut(t: float, b: float, c: float, d: float) -> float:
            """
            正弦缓动函数（入出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            return (-c / 2) * (math.cos((math.pi * t) / d) - 1) + b

    class Expo:
        @staticmethod
        def easeIn(t: float, b: float, c: float, d: float) -> float:
            """
            指数缓动函数（入）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            if t == 0:
                return b
            return c * math.pow(2, 10 * (t / d - 1)) + b

        @staticmethod
        def easeOut(t: float, b: float, c: float, d: float) -> float:
            """
            指数缓动函数（出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            if t == d:
                return b + c
            return c * (-math.pow(2, (-10 * t) / d) + 1) + b

        @staticmethod
        def easeInOut(t: float, b: float, c: float, d: float) -> float:
            """
            指数缓动函数（入出）
            :param t: 当前时间
            :param b: 初始值
            :param c: 变化量
            :param d: 持续时间
            :return: 计算后的数值
            """
            if t == 0:
                return b
            if t == d:
                return b + c
            t /= d / 2
            if t < 1:
                return (c / 2) * math.pow(2, 10 * (t - 1)) + b
            t -= 1
            return (c / 2) * (-math.pow(2, -10 * t) + 2) + b


# 以下是使用示例
if __name__ == "__main__":
    t = 0.5
    b = 0
    c = 100
    d = 1
    print(Tween.Linear(t, b, c, d))
    print(Tween.Quad.easeIn(t, b, c, d))
    print(Tween.Cubic.easeOut(t, b, c, d))